@@ -11,11 +11,11 @@ import datetime
 import logging
 import re
 import sys
-import time
 from decimal import Decimal
 from io import BytesIO
 from ssl import CERT_NONE, CERT_OPTIONAL, CERT_REQUIRED
 
+from e6xdb.server import QueryEngineService
 from thrift.protocol import TBinaryProtocol, TMultiplexedProtocol
 from thrift.transport import TSocket
 from thrift.transport import TTransport
@@ -23,7 +23,6 @@ from thrift.transport import TTransport
 from e6xdb.common import DBAPITypeObject, ParamEscaper, DBAPICursor
 from e6xdb.constants import *
 from e6xdb.datainputstream import DataInputStream, get_query_columns_info, read_rows_from_batch, read_values_from_array
-from e6xdb.server import QueryEngine
 from e6xdb.typeId import *
 
 apilevel = '2.0'
@@ -77,11 +76,11 @@ class HiveParamEscaper(ParamEscaper):
             item = item.decode('utf-8')
         return "'{}'".format(
             item
-                .replace('\\', '\\\\')
-                .replace("'", "\\'")
-                .replace('\r', '\\r')
-                .replace('\n', '\\n')
-                .replace('\t', '\\t')
+            .replace('\\', '\\\\')
+            .replace("'", "\\'")
+            .replace('\r', '\\r')
+            .replace('\n', '\\n')
+            .replace('\t', '\\t')
         )
 
 
@@ -117,8 +116,9 @@ class Connection(object):
         username = username
         password = password
         self.database = database
-        port = port
         scheme = "e6xdb"
+        # service_name = 'E6x'  # E6x  QueryExecutor
+        service_name = 'QueryEngine'  # E6x  QueryExecutor
 
         if scheme != "e6xdb":
             raise ValueError("scheme is not e6xdb")
@@ -132,15 +132,15 @@ class Connection(object):
         self._transport = TTransport.TBufferedTransport(self._transport)
 
         protocol = TBinaryProtocol.TBinaryProtocol(self._transport)
-        protocol = TMultiplexedProtocol.TMultiplexedProtocol(protocol, "E6x")
-        self._client = QueryEngine.Client(protocol)
+        protocol = TMultiplexedProtocol.TMultiplexedProtocol(protocol, service_name)
+        self._client = QueryEngineService.Client(protocol)
         self._transport.open()
 
         try:
             authSession = self._client.authenticate(username, password)
             if not authSession:
                 raise ValueError("invalid username or password")
-            self._client.setSchema(database)
+            # self._client.setSchema(database)
         except Exception as e:
             self._transport.close()
             raise e
@@ -160,7 +160,9 @@ class Connection(object):
         return self._transport.isOpen()
 
     def clear(self, query_id):
-        self._client.clear(query_id)
+        # self._client.clear(query_id)
+        # TODO: Implement clear.
+        pass
 
     def reopen(self):
         self._transport.close()
@@ -215,8 +217,10 @@ class Cursor(DBAPICursor):
         self._data = None
         self._query_columns_description = None
         self._description = None
+        self._query_id = None
         self._batch = list()
         self._rowcount = 0
+        self._database = self.connection.database
 
     def _reset_state(self):
         """Reset state about the previous query in preparation for running another query"""
@@ -280,6 +284,7 @@ class Cursor(DBAPICursor):
         self._query_id = None
         self._batch = None
         self._rowcount = None
+        self._database = None
 
     def get_tables(self):
         schema = self.connection.database
@@ -295,22 +300,29 @@ class Cursor(DBAPICursor):
 
     def cancel(self, query_id):
         _logger.info("Cancelling query")
-        self.connection.query_cancel(self._query_id)
+        self.connection.query_cancel(query_id)
 
     def execute(self, operation, parameters=None, **kwargs):
         """Prepare and execute a database operation (query or command).
         Return values are not defined.
         """
+        """
+        Semicolon is now not supported. So removing it from query end.
+        """
+        if operation.endswith(';'):
+            operation = operation[:-1]
+
         # Prepare statement
         if parameters is None:
             sql = operation
         else:
             sql = operation % _escaper.escape_args(parameters)
 
-        # _logger.info("{}".format(sql))
-
         client = self.connection.client
-        self._query_id = client.prepareStatement(sql)
+        self._query_id = client.prepareStatement(
+            self._database,
+            sql
+        )
         client.executeStatement(self._query_id)
 
         # _logger.info("getting result metadata")
@@ -422,61 +434,43 @@ class Error(Exception):
     pass
 
 
-if __name__ == '__main__':
-    connection = connect(host="localhost",
-                         port=9000,
-                         scheme="e6xdb",
-                         username="admin",
-                         database="tpcds_10gb",
-                         auth=None,
-                         configuration=None,
-                         kerberos_service_name=None,
-                         password="admin",
-                         check_hostname=None,
-                         ssl_cert=None,
-                         thrift_transport=None)
-    cursor = Cursor(connection)
-    cursor.execute("select * from store_sales LEFT OUTER JOIN store_returns ON (sr_item_sk = ss_item_sk AND sr_ticket_number = ss_ticket_number);")
-    bdata = []
-    try:
-        for i in range(1000):
-            cursor.cancel(None)
-            bdata += cursor.fetchmany()
-        print('total rows in the batch resultset ', len(bdata))
-        r_count = cursor.rowcount()
-        print('total row count', r_count)
-    except Exception as e:
-        print('Error Message ', e.__cause__)
-
-    while True:
-        try:
-            connection = connect(host="localhost",
-                                 port=9000,
-                                 scheme="e6xdb",
-                                 username="admin",
-                                 database="tpcds_10gb",
-                                 auth=None,
-                                 configuration=None,
-                                 kerberos_service_name=None,
-                                 password="admin",
-                                 check_hostname=None,
-                                 ssl_cert=None,
-                                 thrift_transport=None)
-            cursor = Cursor(connection)
-            isOpen = cursor.connection.check_connection
-            if not isOpen:
-                time.sleep(5)
-                continue
-            else:
-                break
-        except:
-            time.sleep(5)
-            continue
-
-    cursor.execute("SELECT * from catalog_sales limit 100;")
-    bdata = []
-    for i in range(1000):
-        bdata += cursor.fetchmany()
+# if __name__ == '__main__':
+#     host = '18.212.241.241'
+#     e6x_connection = connect(
+#         host=host,
+#         port=9001,
+#         scheme="e6xdb",
+#         username="admin",
+#         database="tpcds_1000",
+#         auth=None,
+#         configuration=None,
+#         kerberos_service_name=None,
+#         password="admin",
+#         check_hostname=None,
+#         ssl_cert=None,
+#         thrift_transport=None
+#     )
+#     print('Connected.')
+#     import time
+#     start = time.time()
+#     cursor = e6x_connection.cursor()
+#     query = 'SELECT * FROM date_dim limit 1;'
+#     cursor.execute(query)
+#     all_records = cursor.fetchall()
+#     print(time.time() - start)
+    # print(all_records)
+    # e6x_connection.close()
+#
+#     e6x_connection.reopen()
+#
+#     cursor = e6x_connection.cursor()
+#     query = 'SELECT * FROM date_dim limit 4;'
+#     cursor.execute(query)
+#     all_records = cursor.fetchall()
+#     print(all_records)
+#     # cursor.close()
+#
+#     e6x_connection.close()
 
 #
 # Type Objects and Constructors
